var Immutable, React, classnames, div, input, locale, pre, span;

React = require("react");

Immutable = require("immutable");

classnames = require("classnames");

locale = require('./locale');

div = React.createFactory("div");

pre = React.createFactory("pre");

input = React.createFactory("input");

span = document.createElement('span');

module.exports = React.createClass({
  displayName: "actions-recorder-controller",
  propTypes: {
    records: React.PropTypes.instanceOf(Immutable.List),
    inital: React.PropTypes.instanceOf(Immutable.Map),
    updater: React.PropTypes.func.isRequired,
    pointer: React.PropTypes.number.isRequired,
    isTravelling: React.PropTypes.bool.isRequired,
    onCommit: React.PropTypes.func.isRequired,
    onReset: React.PropTypes.func.isRequired,
    onPeek: React.PropTypes.func.isRequired,
    onMergeBefore: React.PropTypes.func.isRequired,
    onClearAfter: React.PropTypes.func.isRequired,
    onRun: React.PropTypes.func.isRequired,
    language: React.string
  },
  getDefaultProps: function() {
    return {
      language: 'en',
      isDragging: false,
      x: 100,
      y: 100
    };
  },
  getInitialState: function() {
    return {
      dataPath: ''
    };
  },
  componentDidMount: function() {
    return window.addEventListener('dragover', this.onMove);
  },
  componentWillUnmount: function() {
    return window.removeEventListener('dragover', this.onMove);
  },
  onChange: function(event) {
    return this.setState({
      dataPath: event.target.value
    });
  },
  onDragStart: function(event) {
    event.dataTransfer.setDragImage(span, 0, 0);
    return this.setState({
      isDragging: true
    });
  },
  onDragEnd: function(event) {
    return this.setState({
      isDragging: false
    });
  },
  onMove: function(event) {
    return this.setState({
      x: event.pageX - 400,
      y: event.pageY - 250
    });
  },
  renderItem: function(record, index) {
    var className, onClick;
    onClick = (function(_this) {
      return function() {
        return _this.props.onPeek(index);
      };
    })(this);
    className = classnames("recorder-item", {
      "is-pointer": this.props.isTravelling && this.props.pointer === index
    });
    return div({
      className: className,
      onClick: onClick
    }, record.get(0));
  },
  renderAction: function() {
    var record;
    record = this.props.records.get(this.props.pointer);
    return pre({
      className: 'recorder-action-data'
    }, JSON.stringify(record.get(1), null, 2));
  },
  renderPrev: function() {
    var result, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    result = this.props.records.slice(0, this.props.pointer).butLast().reduce(updater, this.props.initial);
    return pre({
      className: 'recorder-prev'
    }, this.renderResult(result));
  },
  renderStore: function() {
    var result, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    result = this.props.records.slice(0, this.props.pointer).reduce(updater, this.props.initial);
    return pre({
      className: 'recorder-store'
    }, this.renderResult(result));
  },
  renderResult: function(result) {
    var dataPath, helper;
    dataPath = this.state.dataPath.split(/\s+/).filter(function(chunk) {
      return chunk.length > 0;
    });
    helper = function(data, keys) {
      if (keys.length === 0) {
        return data;
      } else {
        try {
          return helper(data.get(keys[0]), keys.slice(1));
        } catch (_error) {}
      }
    };
    return JSON.stringify(helper(result, dataPath), null, 2);
  },
  render: function() {
    return div({
      className: "recorder-controller",
      draggable: true,
      onDragStart: this.onDragStart,
      onDragEnd: this.onDragEnd,
      style: {
        top: this.state.y,
        left: this.state.x
      }
    }, div({
      className: "recorder-header line"
    }, div({
      className: "button is-attract",
      onClick: this.props.onCommit
    }, locale.get('commit', this.props.language)), div({
      className: "button is-danger",
      onClick: this.props.onReset
    }, locale.get('reset', this.props.language)), div({
      className: "button is-attract",
      onClick: this.props.onMergeBefore
    }, locale.get('mergeBefore', this.props.language)), div({
      className: "button is-danger",
      onClick: this.props.onClearAfter
    }, locale.get('clearAfter', this.props.language)), this.props.isTravelling ? div({
      className: "button is-attract",
      onClick: this.props.onRun
    }, locale.get('run', this.props.language)) : void 0, input({
      value: this.state.dataPath,
      onChange: this.onChange
    })), div({
      className: 'recorder-viewer'
    }, div({
      className: "recorder-monitor"
    }, this.props.records.map(this.renderItem)), this.props.isTravelling && (this.props.records.get(this.props.pointer) != null) ? div({
      className: "recorder-details"
    }, this.renderPrev(), this.renderAction(), this.renderStore()) : void 0));
  }
});
